@@ -1,13 +1,11 @@
 from ._core.dispatcher.bus import Bus
 from ._core.dispatcher.pipe import Pipe
-from ._core.dto import DTO
 from ._core.message import (
     AnyCommandBus,
     Command,
     CommandBus,
     Event,
     EventBus,
-    Message,
     Query,
     QueryBus,
     command_handler,
@@ -27,10 +25,8 @@ __all__ = (
     "CQScope",
     "Command",
     "CommandBus",
-    "DTO",
     "Event",
     "EventBus",
-    "Message",
     "Middleware",
     "MiddlewareResult",
     "Pipe",
