@@ -33,18 +33,22 @@ query_handler: HandlerDecorator[Query, Any] = HandlerDecorator(
 )
 
 
-@injection.singleton(inject=False, mode="fallback")
+@injection.injectable(inject=False, mode="fallback")
 def new_command_bus() -> CommandBus:  # type: ignore[type-arg]
     bus = SimpleBus(command_handler.manager)
-    bus.add_middlewares(InjectionScopeMiddleware(CQScope.ON_COMMAND))
+    transaction_scope_middleware = InjectionScopeMiddleware(
+        CQScope.TRANSACTION,
+        exist_ok=True,
+    )
+    bus.add_middlewares(transaction_scope_middleware)
     return bus
 
 
-@injection.singleton(inject=False, mode="fallback")
+@injection.injectable(inject=False, mode="fallback")
 def new_event_bus() -> EventBus:
     return TaskBus(event_handler.manager)
 
 
-@injection.singleton(inject=False, mode="fallback")
+@injection.injectable(inject=False, mode="fallback")
 def new_query_bus() -> QueryBus:  # type: ignore[type-arg]
     return SimpleBus(query_handler.manager)
